﻿Imports ThinkGeo.MapSuite.Core
Imports ThinkGeo.MapSuite.DesktopEdition
Imports System.Collections.ObjectModel
Imports System.Drawing.Printing
Imports System.IO
Imports System.ComponentModel
Imports System.Runtime.Serialization.Formatters.Binary
Imports System.Data.SqlClient

Public Class Form1
   Private moRoadsLayer As ShapeFileFeatureLayer
   Private moHydroLayer As ShapeFileFeatureLayer
   Private moPolyLayer As ShapeFileFeatureLayer
   Private moFeatureExtent As RectangleShape
   Private mbStyle1 As Boolean = True
   Private moLegendForm As frmLegendProps
   Public Enum EnumPagePosition
      TopLeft = 1
      TopRight = 2
      LowerLeft = 3
      LowerRight = 4
   End Enum
   Public Enum EnumCenterType
      Horizontal = 1
      Vertical = 2
   End Enum
   Private Sub AddCenteredTitleLabel()
      Dim oLabelPrinterLayer As LabelPrinterLayer = New LabelPrinterLayer()
      Dim oPageBoundingbox As RectangleShape

      '***** Setup the text and the font..
      With oLabelPrinterLayer
         .Text = "Page Layout"
         .Font = New GeoFont("Arial", 10, DrawingFontStyles.Bold)
         .TextBrush = New GeoSolidBrush(GeoColor.StandardColors.Black)

         '***** Set the labels position so that is it centered on the page one inch from the top                        
         oPageBoundingbox = GetPageBoundingBox(PrintingUnit.Inch)
         Dim oLabelCenter As PointShape = New PointShape()
         oLabelCenter.X = oPageBoundingbox.UpperRightPoint.X - oPageBoundingbox.Width / 2
         oLabelCenter.Y = oPageBoundingbox.UpperLeftPoint.Y - 0.75

         .PrinterWrapMode = PrinterWrapMode.AutoSizeText
         .ResizeMode = PrinterResizeMode.MaintainAspectRatio
         .SetPosition(5, 0.5, oLabelCenter, PrintingUnit.Inch)
      End With
      Dim oPrinterOverlay As PrinterInteractiveOverLay = mGetPrinterOverlay()
      oPrinterOverlay.PrinterLayers.Add(Guid.NewGuid.ToString, oLabelPrinterLayer)

   End Sub

   Private Sub Form1_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load

      CreateDefaultLayout()
      moLegendForm = New frmLegendProps(Me, PageLayout)
      moLegendForm.Show()

   End Sub

   Public Sub CreateDefaultLayout()
      Dim oExtentRectangle As RectangleShape
      Dim moPrinterOverlay As PrinterInteractiveOverLay
      Dim moPrinterPage As PagePrinterLayer

      moPrinterOverlay = New PrinterInteractiveOverLay()
      With PageLayout
         '***** Setup the map unit, you want to always use feet or meters for printer layout maps
         .MapUnit = GeographyUnit.Feet
         '***** Create the default zoom levels for the layout map
         .ZoomLevelSet = New PrinterZoomLevelSet(.MapUnit, PrinterHelper.GetPointsPerGeographyUnit(.MapUnit))

         '***** Set the background color to gray so there is contrast with the white page
         .BackgroundOverlay.BackgroundBrush = New GeoSolidBrush(GeoColor.StandardColors.LightGray)

         '***** Create the PrinterInteractiveOverlay to contain all of the PrinterLayers.
         '***** The interactive overlay allows the layers to be interacted with
         .InteractiveOverlays.Add("PrinterOverlay", moPrinterOverlay)
         .InteractiveOverlays.MoveToBottom("PrinterOverlay")

         '***** Create the PagePrinterLayer which shows the page boundary and 
         '***** is the area the user will arrange all of the other layers on top of.
         moPrinterPage = New PagePrinterLayer()
         With moPrinterPage
            .Open()
            .BackgroundMask = AreaStyles.CreateSimpleAreaStyle(GeoColor.StandardColors.White, GeoColor.SimpleColors.Black, 1)
            moPrinterOverlay.PrinterLayers.Add("PrinterPage", moPrinterPage)
            '***** Get the page's extent, scale it up by 10%
            oExtentRectangle = CType(RectangleShape.ScaleUp(.GetPosition(), 10), RectangleShape)
            '***** Set oExtentRectangle as the default current extent
         End With
         .CurrentExtent = oExtentRectangle

         '***** Set the minimum scale of the map to the last zoom level
         .MinimumScale = .ZoomLevelSet.ZoomLevel20.Scale

         '***** Add a container for the map layers
         AddMapLayerContainer(mGetPageLayoutMap)

         '***** Add a layer to the page layout
         AddMapLayers(mGetPageLayoutMap)

         '***** Add a scale line
         ''CreateDefaultScaleLine(mGetPageLayoutMap)

         '***** Add a scale bar
         'CreateDefaultScaleBar(mGetPageLayoutMap)

         CreateDefaultMagDecLayer(mGetPageLayoutMap)

         '***** A refresh is done in the form's Shown event
         'AddLegend()
         AddCenteredTitleLabel()
         'AddImage()
      End With
   End Sub
   Public Sub AddMapLayers(moPageLayoutMap As MapPrinterLayer)
      moPageLayoutMap.MapUnit = GeographyUnit.Meter


      Dim sPath As String = "..\..\Data\AlabamaCounties.shp"

      'Dim oSqlLayer As New MsSql2008FeatureLayer(sConnect, "Counties", "OBJECTID", 4267)
      Dim oShapeFileLayer As New ShapeFileFeatureLayer(sPath)
      moPageLayoutMap.Layers.Add(oShapeFileLayer)
      Dim oColor As New GeoColor(100, GeoColor.GeographicColors.Sand)
      oShapeFileLayer.ZoomLevelSet.ZoomLevel01.DefaultTextStyle = New TextStyle("Name", New GeoFont("Arial", 10), New GeoSolidBrush(GeoColor.SimpleColors.Black))
      oShapeFileLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = AreaStyles.CreateSimpleAreaStyle(oColor, GeoColor.SimpleColors.Blue, 4)
      oShapeFileLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20

      'Dim oProj As New ManagedProj4Projection(ManagedProj4Projection.GetEsriParametersString(102003), ManagedProj4Projection.GetEsriParametersString(26916))
      'oShapeFileLayer.FeatureSource.Projection = oProj

      oShapeFileLayer.Open()
      'Dim oFeatures As Collection(Of Feature) = oSqlLayer.QueryTools.GetFeaturesByColumnValue("State_Name", "Nebraska")
      'Dim oFeatures As Collection(Of Feature) = oShapeFileLayer.QueryTools.GetFeaturesByColumnValue("StateName", "Alabama")
      Dim oExtent As RectangleShape = oShapeFileLayer.GetBoundingBox
      moPageLayoutMap.MapExtent = oExtent

   End Sub
   Public Sub CreateDefaultMagDecLayer(moPageLayoutMap As MapPrinterLayer)

      Dim moPrinterOverlay As PrinterInteractiveOverLay = mGetPrinterOverlay()
      If moPrinterOverlay.PrinterLayers.Contains("MagDecLayer") Then
         moPrinterOverlay.PrinterLayers.Remove("MagDecLayer")
      End If
      Dim moMagDecLayer As MagneticDeclinationPrinterLayer

      '***** Create the scaleLinePrinterLayer and pass in the MapPrinterLayer
      moMagDecLayer = New MagneticDeclinationPrinterLayer(moPageLayoutMap)

      Dim oProj As New Proj4Projection()
      oProj.ExternalProjectionParametersString = Proj4Projection.GetGoogleMapParametersString()
      oProj.InternalProjectionParametersString = Proj4Projection.GetEsriParametersString(26916)
      oProj.Open()
      ''moMagDecLayer.ProjectionToSphericalMercator = oProj

      moMagDecLayer.Open()
      'moMagDecLayer.DragMode = PrinterDragMode.Draggable
      'moMagDecLayer.ResizeMode = PrinterResizeMode.MaintainAspectRatio
      mSetPosition(2, 2, moMagDecLayer, EnumPagePosition.LowerLeft, mGetPageLayoutMap)

      '***** Add the ScaleLinePrinterLayer to the PrinterInteractiveOverlay
      moPrinterOverlay.PrinterLayers.Add("MagDecLayer", moMagDecLayer)
   End Sub

   Public Sub SetScale(moPageLayoutMap As MapPrinterLayer)
      Dim dWidth As Single = CSng(moPageLayoutMap.GetPosition(PrintingUnit.Inch).Width * 96)
      Dim dHeight As Single = CSng(moPageLayoutMap.GetPosition(PrintingUnit.Inch).Height * 96)
      Dim oExtent As RectangleShape = ExtentHelper.ZoomToScale(15840, moPageLayoutMap.MapExtent, moPageLayoutMap.MapUnit, dWidth, dHeight)
      oExtent = ExtentHelper.CenterAt(oExtent, moPageLayoutMap.MapExtent.GetCenterPoint, dWidth, dHeight)
      moPageLayoutMap.MapExtent = oExtent
      btnRefresh_Click(Nothing, Nothing)
   End Sub
   Public Sub AddMapLayerContainer(moPageLayoutMap As MapPrinterLayer)
      Dim oPageBoundingbox As RectangleShape = GetPageBoundingBox(PrintingUnit.Inch)

      moPageLayoutMap = New MapPrinterLayer()

      moPageLayoutMap.MapUnit = GeographyUnit.Feet

      '***** Create the MapPrinterLayer and set the position
      moPageLayoutMap.BackgroundMask = AreaStyles.CreateSimpleAreaStyle(GeoColor.StandardColors.White, GeoColor.SimpleColors.Black, 1)

      '***** Set the maps position slightly below the pages center and 7.5 inches wide and 9 inches tall
      oPageBoundingbox = GetPageBoundingBox(PrintingUnit.Inch)
      moPageLayoutMap.SetPosition(7.5, 9, oPageBoundingbox.GetCenterPoint().X, oPageBoundingbox.GetCenterPoint().Y - 0.25, PrintingUnit.Inch)

      '***** Add the MapPrinterLayer to the PrinterInteractiveOverlay
      Dim moPrinterOverlay As PrinterInteractiveOverLay = mGetPrinterOverlay()
      moPrinterOverlay.PrinterLayers.Add("MapPrinterLayer", moPageLayoutMap)
   End Sub
   Private Function GetPageBoundingBox(ByVal unit As PrintingUnit) As RectangleShape
      Dim moPrinterPage As PagePrinterLayer = mGetPrinterPage()
      Return moPrinterPage.GetPosition(unit)
   End Function
   Public Sub CreateDefaultScaleLine(moPageLayoutMap As MapPrinterLayer)
      Dim moPrinterOverlay As PrinterInteractiveOverLay = mGetPrinterOverlay()
      If moPrinterOverlay.PrinterLayers.Contains("ScaleLine") Then
         moPrinterOverlay.PrinterLayers.Remove("ScaleLine")
      End If
      Dim moScaleLineLayer As ScaleLinePrinterLayer
      '***** Create the scaleLinePrinterLayer and pass in the MapPrinterLayer
      moScaleLineLayer = New ScaleLinePrinterLayer(moPageLayoutMap)

      moScaleLineLayer.MapUnit = moPageLayoutMap.MapUnit
      ''moScaleLineLayer.IsVisible = False
      moScaleLineLayer.Open()
      mSetPosition(2.5, 0.25, moScaleLineLayer, EnumPagePosition.LowerLeft, mGetPageLayoutMap)

      '***** Add the ScaleLinePrinterLayer to the PrinterInteractiveOverlay
      moPrinterOverlay.PrinterLayers.Add("ScaleLine", moScaleLineLayer)
   End Sub

   Public Sub CreateDefaultScaleBar(moPageLayoutMap As MapPrinterLayer)
      Dim moPrinterOverlay As PrinterInteractiveOverLay = mGetPrinterOverlay()
      If moPrinterOverlay.PrinterLayers.Contains("ScaleBar") Then
         moPrinterOverlay.PrinterLayers.Remove("ScaleBar")
      End If

      '***** Create the scaleLinePrinterLayer and pass in the MapPrinterLayer
      Dim moScaleBarLayer As ScaleBarPrinterLayer
      moScaleBarLayer = New ScaleBarPrinterLayer(moPageLayoutMap)

      With moScaleBarLayer
         .MapUnit = moPageLayoutMap.MapUnit
         .AlternateBarBrush = New GeoSolidBrush(GeoColor.SimpleColors.Black)
         .TextColor = GeoColor.SimpleColors.Black
         .Font = New GeoFont("Arial", 10)
         .BarBrush = New GeoSolidBrush(GeoColor.SimpleColors.BrightOrange)
         .UnitFamily = UnitSystem.Imperial
         .MaxWidth = 7
      End With

      moScaleBarLayer.Open()
      mSetPosition(2.5, 0.25, moScaleBarLayer, EnumPagePosition.LowerRight, mGetPageLayoutMap)

      '***** Add the ScaleLinePrinterLayer to the PrinterInteractiveOverlay
      moPrinterOverlay.PrinterLayers.Add("ScaleBar", moScaleBarLayer)
   End Sub
   Public Sub mSetPosition(ByVal dWidth As Double, ByVal dHeight As Double, ByVal oPrinterLayer As PrinterLayer, ByVal eInitialPosition As EnumPagePosition, moPageLayoutMap As MapPrinterLayer)
      Dim dCenterPointX As Double
      Dim dCenterPointY As Double
      Dim dDistanceFromEdge As Double = 0.25

      '***** Get the bounding box for the MapPrinterLayer
      moPageLayoutMap.Open()
      Dim oMapBoundingBox As RectangleShape = moPageLayoutMap.GetPosition(PrintingUnit.Inch)
      Select Case eInitialPosition
         Case EnumPagePosition.LowerRight
            dCenterPointX = oMapBoundingBox.LowerRightPoint.X - (dWidth / 2) - dDistanceFromEdge
            dCenterPointY = oMapBoundingBox.LowerRightPoint.Y + (dHeight / 2) + dDistanceFromEdge
         Case EnumPagePosition.LowerLeft
            dCenterPointX = oMapBoundingBox.LowerLeftPoint.X + (dWidth / 2) + dDistanceFromEdge
            dCenterPointY = oMapBoundingBox.LowerLeftPoint.Y + (dHeight / 2) + dDistanceFromEdge
         Case EnumPagePosition.TopRight
            dCenterPointX = oMapBoundingBox.UpperRightPoint.X - (dWidth / 2) - dDistanceFromEdge
            dCenterPointY = oMapBoundingBox.UpperRightPoint.Y - (dHeight / 2) - dDistanceFromEdge
         Case EnumPagePosition.TopLeft
            dCenterPointX = oMapBoundingBox.UpperLeftPoint.X + (dWidth / 2) + dDistanceFromEdge
            dCenterPointY = oMapBoundingBox.UpperLeftPoint.Y - (dHeight / 2) - dDistanceFromEdge
      End Select

      '***** Now let's try to place this layer in the lower right corner of the page
      oPrinterLayer.SetPosition(dWidth, dHeight, dCenterPointX, dCenterPointY, PrintingUnit.Inch)
   End Sub

   Private Sub btnRefresh_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnRefresh.Click
      mRefreshMapLayers(mGetPageLayoutMap)
   End Sub

   Private Sub mRefreshMapLayers(moPageLayoutMap As MapPrinterLayer)
      If Not moPageLayoutMap Is Nothing Then
         moPageLayoutMap.IsDrawing = True
         ''moPageLayoutMap.ClearCache()
      End If

      PageLayout.Refresh()

      If Not moPageLayoutMap Is Nothing Then
         moPageLayoutMap.IsDrawing = False
      End If
   End Sub
   Private Sub mCenterElement(ByVal eCenterType As EnumCenterType, ByVal oPrinterLayer As PrinterLayer)
      Dim oCurPosition As RectangleShape = Nothing
      Dim oNewCenterPoint As PointShape

      oPrinterLayer.Open()
      oCurPosition = oPrinterLayer.GetPosition(PrintingUnit.Inch)

      Dim oPageBoundingBox As RectangleShape = GetPageBoundingBox(PrintingUnit.Inch)


      If eCenterType = EnumCenterType.Horizontal Then
         oNewCenterPoint = New PointShape(0, oCurPosition.GetCenterPoint.Y)
      Else
         oNewCenterPoint = New PointShape(oCurPosition.GetCenterPoint.X, 0)
      End If

      oPrinterLayer.SetPosition(oCurPosition.Width, oCurPosition.Height, oNewCenterPoint, PrintingUnit.Inch)


      mRefreshMapLayers(mGetPageLayoutMap)
   End Sub


   Private Sub btnAddScaleline_Click(sender As Object, e As System.EventArgs) Handles btnAddScaleline.Click
      If btnAddScaleline.Text = "Add Scaleline" Then
         CreateDefaultScaleLine(mGetPageLayoutMap)
         btnAddScaleline.Text = "Remove Scaleline"
      Else
         '***** Add the ScaleLinePrinterLayer to the PrinterInteractiveOverlay
         mGetPrinterOverlay.PrinterLayers.Remove("ScaleLine")
         btnAddScaleline.Text = "Add Scaleline"
      End If

      PageLayout.Refresh()
   End Sub

   Private Sub Form1_Shown(sender As Object, e As System.EventArgs) Handles Me.Shown
      'btnRefresh_Click(Nothing, Nothing)
      Me.PageLayout.Refresh()

   End Sub

   Public Function GetUserFolder() As String
      Dim sBasePath As String
      Dim sApplicationPath As String

      sBasePath = Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData)
      sApplicationPath = Application.CompanyName & "\" & Application.ProductName & "\"

      If Not System.IO.Directory.Exists(sBasePath & "\" & sApplicationPath) Then
         System.IO.Directory.CreateDirectory(sBasePath & "\" & sApplicationPath)
      End If

      Return sBasePath & "\" & sApplicationPath

   End Function
   Private Function mGetPrinterOverlay() As PrinterInteractiveOverLay
      For Each oPrinterOverlay In PageLayout.InteractiveOverlays
         If TypeOf oPrinterOverlay Is PrinterInteractiveOverLay Then
            Return CType(oPrinterOverlay, PrinterInteractiveOverLay)
         End If
      Next oPrinterOverlay
      Return Nothing
   End Function
   Private Function mGetPageLayoutMap() As MapPrinterLayer
      Dim oPrinterOverlay As PrinterInteractiveOverLay = mGetPrinterOverlay()
      Dim oPrinterLayer As PrinterLayer
      For Each oPrinterLayer In oPrinterOverlay.PrinterLayers
         If TypeOf oPrinterLayer Is MapPrinterLayer Then
            Return CType(oPrinterLayer, MapPrinterLayer)
         End If
      Next oPrinterLayer
      Return Nothing
   End Function
   Private Function mGetPrinterPage() As PagePrinterLayer
      Dim oPrinterOverlay As PrinterInteractiveOverLay = mGetPrinterOverlay()
      Dim oPrinterLayer As PrinterLayer
      For Each oPrinterLayer In oPrinterOverlay.PrinterLayers
         If TypeOf oPrinterLayer Is PagePrinterLayer Then
            Return CType(oPrinterLayer, PagePrinterLayer)
         End If
      Next oPrinterLayer
      Return Nothing
   End Function
   Public Sub Print()

      ''***** Get the printer for printing
      'Dim oPrinterSettings As PrinterSettings = GetPrinter()
      'If oPrinterSettings Is Nothing Then
      '   Return
      'End If

      'Dim oPrinterOverLay As PrinterInteractiveOverLay = mGetPrinterOverlay()
      'Dim oPagePrinterLayer As PagePrinterLayer = mGetPrinterPage()

      'Dim oPrintDocument As New PrintDocument()
      'With oPrintDocument
      '   .PrinterSettings = oPrinterSettings
      '   .DefaultPageSettings.Landscape = True
      '   If oPagePrinterLayer.Orientation = PrinterOrientation.Portrait Then
      '      .DefaultPageSettings.Landscape = False
      '   End If
      '   .DefaultPageSettings.PaperSize = GetPrintPreviewPaperSize(oPagePrinterLayer)
      'End With

      'Dim oPrinterCanvas As PrinterGeoCanvas = New PrinterGeoCanvas()

      ''***** Create a rectangle to represent the drawing area for the document
      'oPrinterCanvas.DrawingArea = mGetPrinterCanvasDrawingArea(oPrintDocument)
      ''***** Start the drawing process
      'oPrinterCanvas.BeginDrawing(oPrintDocument, oPagePrinterLayer.GetBoundingBox(), PageLayout.MapUnit)

      ''***** Loop through all of the PrintingLayers in the PrinterInteractiveOverlay 
      ''***** and print all of the except for the PagePrinterLayer                
      'Dim labelsInAllLayers As Collection(Of SimpleCandidate) = New Collection(Of SimpleCandidate)()
      'For Each oPrinterLayer As PrinterLayer In oPrinterOverLay.PrinterLayers
      '   If TypeOf oPrinterLayer Is MapPrinterLayer Then
      '      CType(oPrinterLayer, MapPrinterLayer).ClearCache()
      '   End If

      '   oPrinterLayer.IsDrawing = True
      '   If Not (TypeOf oPrinterLayer Is PagePrinterLayer) Then
      '      oPrinterLayer.Draw(oPrinterCanvas, labelsInAllLayers)
      '   End If
      '   oPrinterLayer.IsDrawing = False
      'Next oPrinterLayer

      'oPrinterCanvas.EndDrawing()


      '***** Get the printer for printing
      Dim oPrinterSettings As PrinterSettings = GetPrinter()
      If oPrinterSettings Is Nothing Then
         Return
      End If

      Dim oPrinterOverLay As PrinterInteractiveOverLay = mGetPrinterOverlay()
      Dim oPagePrinterLayer As PagePrinterLayer = mGetPrinterPage()

      Dim oPrintDocument As New PrintDocument()
      With oPrintDocument
         .PrinterSettings = oPrinterSettings
         .DefaultPageSettings.Landscape = True
         If oPagePrinterLayer.Orientation = PrinterOrientation.Portrait Then
            .DefaultPageSettings.Landscape = False
         End If
         .DefaultPageSettings.PaperSize = GetPrintPreviewPaperSize(oPagePrinterLayer)
      End With

      Dim oPrinterCanvas As PrinterGeoCanvas = New PrinterGeoCanvas()

      '***** Create a rectangle to represent the drawing area for the document
      oPrinterCanvas.DrawingArea = mGetPrinterCanvasDrawingArea(oPrintDocument)
      '***** Start the drawing process
      oPrinterCanvas.BeginDrawing(oPrintDocument, oPagePrinterLayer.GetBoundingBox(), PageLayout.MapUnit)

      '***** Loop through all of the PrintingLayers in the PrinterInteractiveOverlay 
      '***** and print all of the except for the PagePrinterLayer                
      Dim labelsInAllLayers As Collection(Of SimpleCandidate) = New Collection(Of SimpleCandidate)()
      For Each oPrinterLayer As PrinterLayer In oPrinterOverLay.PrinterLayers
         oPrinterLayer.IsDrawing = True


         If Not (TypeOf oPrinterLayer Is PagePrinterLayer) Then
            oPrinterLayer.Draw(oPrinterCanvas, labelsInAllLayers)
         End If


         oPrinterLayer.IsDrawing = False
      Next oPrinterLayer

      oPrinterCanvas.EndDrawing()
   End Sub
   Private Sub btnExportToPdf_Click(ByVal sender As Object, ByVal e As EventArgs)
      ' Get the PrinterInteractiveOverlay and PagePrinterLayer as we are going to need them below
      Dim printerInteractiveOverLay As PrinterInteractiveOverLay = mGetPrinterOverlay() ' CType(PageLayout.InteractiveOverlays("PrintPreviewOverlay"), PrinterInteractiveOverLay)
      Dim pagePrinterLayer As PagePrinterLayer = mGetPrinterPage() ' TryCast(printerInteractiveOverLay.PrinterLayers("PageLayer"), PagePrinterLayer)

      Dim pdfDocument As PdfSharp.Pdf.PdfDocument = Nothing
      Dim pdfPage As PdfSharp.Pdf.PdfPage = Nothing

      Try
         ' Create the PDF documents, pages, and copnfigure them
         pdfDocument = New PdfSharp.Pdf.PdfDocument()
         pdfPage = pdfDocument.AddPage()
         If pagePrinterLayer.Orientation = PrinterOrientation.Portrait Then
            pdfPage.Orientation = PdfSharp.PageOrientation.Portrait
         Else
            pdfPage.Orientation = PdfSharp.PageOrientation.Landscape
         End If
         pdfPage.Size = GetPdfPageSize(pagePrinterLayer.PageSize)

         ' Create the PdfGeoCanvas and pass tha PdfPage into the BeginDrawing 
         Dim pdfGeoCanvas As PdfGeoCanvas = New PdfGeoCanvas()
         pdfGeoCanvas.BeginDrawing(pdfPage, pagePrinterLayer.GetBoundingBox(), PageLayout.MapUnit)

         ' Loop through all of the PrinterLayers and draw them to the canvas except for the
         ' PagePrinterLayer which is just used for the visual layout
         Dim labelsInAllLayers As Collection(Of SimpleCandidate) = New Collection(Of SimpleCandidate)()
         For Each printerLayer As PrinterLayer In printerInteractiveOverLay.PrinterLayers
            printerLayer.IsDrawing = True
            If Not (TypeOf printerLayer Is PagePrinterLayer) Then
               printerLayer.Draw(pdfGeoCanvas, labelsInAllLayers)
            End If
            printerLayer.IsDrawing = False
         Next printerLayer

         ' Finish up the drawing
         pdfGeoCanvas.EndDrawing()

         ' Dave the PDF document and launch it in the default viewer to show the user
         Dim filename As String = "PrintingResults.pdf"
         pdfDocument.Save(filename)
         Process.Start(filename)
      Catch ex As Exception
         MessageBox.Show(ex.Message & " " & ex.StackTrace, "Exception")
      Finally
         If Not pdfDocument Is Nothing Then
            pdfDocument.Dispose()
         End If
      End Try
   End Sub

   Private Function mGetPrinterCanvasDrawingArea(oPrintDocument As PrintDocument) As Rectangle
      Dim oRectangle As Rectangle
      '***** Here we create a System.Drawing.Rectangle to be used as the drawing
      '***** area for the PrinterGeoCanvas that will draw the map on the printer.
      '***** Notice that when the page has been set to Landscape the Width and Height
      '***** must be reversed.
      With oPrintDocument.DefaultPageSettings.PrintableArea()
         If oPrintDocument.DefaultPageSettings.Landscape Then
            oRectangle = New Rectangle(0, 0, Convert.ToInt32(.Height), Convert.ToInt32(.Width))
         Else
            oRectangle = New Rectangle(0, 0, Convert.ToInt32(.Width), Convert.ToInt32(.Height))
         End If
      End With

      Return oRectangle
   End Function

   Public Shared Function GetPrinter() As PrinterSettings
      Dim oPrintDialog As New PrintDialog
      Dim oPrinterSettings As PrinterSettings = Nothing

      With oPrintDialog
         If Not .ShowDialog() = DialogResult.Cancel Then
            oPrinterSettings = .PrinterSettings
         End If
      End With
      Return oPrinterSettings
   End Function
   Private Function GetPrintPreviewPaperSize(ByVal pagePrinterLayer As PagePrinterLayer) As PaperSize
      Dim printPreviewPaperSize As PaperSize = New PaperSize("AnsiA", 850, 1100)
      Select Case pagePrinterLayer.PageSize
         Case PrinterPageSize.AnsiA
            printPreviewPaperSize = New PaperSize("AnsiA", 850, 1100)
         Case PrinterPageSize.AnsiB
            printPreviewPaperSize = New PaperSize("AnsiB", 1100, 1700)
         Case PrinterPageSize.AnsiC
            printPreviewPaperSize = New PaperSize("AnsiC", 1700, 2200)
         Case PrinterPageSize.AnsiD
            printPreviewPaperSize = New PaperSize("AnsiD", 2200, 3400)
         Case PrinterPageSize.AnsiE
            printPreviewPaperSize = New PaperSize("AnsiE", 3400, 4400)
         Case PrinterPageSize.Custom
            printPreviewPaperSize = New PaperSize("Custom Size", CInt(pagePrinterLayer.CustomWidth), CInt(pagePrinterLayer.CustomHeight))
         Case Else
      End Select

      Return printPreviewPaperSize
   End Function

   Private Sub btnPrint_Click(sender As System.Object, e As System.EventArgs) Handles btnPrint.Click
      btnPrint.Enabled = False
      Print()
      btnPrint.Enabled = True
   End Sub

   Public Sub New()

      ' This call is required by the designer.
      InitializeComponent()

      ' Add any initialization after the InitializeComponent() call.

   End Sub
   Private Function GetPdfPageSize(ByVal pageSize As PrinterPageSize) As PdfSharp.PageSize
      Dim pdfPageSize As PdfSharp.PageSize = PdfSharp.PageSize.Letter
      Select Case pageSize
         Case PrinterPageSize.AnsiA
            pdfPageSize = PdfSharp.PageSize.Letter
         Case PrinterPageSize.AnsiB
            pdfPageSize = PdfSharp.PageSize.Ledger
         Case PrinterPageSize.AnsiC
            pdfPageSize = PdfSharp.PageSize.A2
         Case PrinterPageSize.AnsiD
            pdfPageSize = PdfSharp.PageSize.A1
         Case PrinterPageSize.AnsiE
            pdfPageSize = PdfSharp.PageSize.A0
         Case PrinterPageSize.Custom
            Throw New NotSupportedException()
         Case Else
            Throw New NotSupportedException()
      End Select
      Return pdfPageSize
   End Function

   Private Sub PageLayout_MouseUp(sender As Object, e As System.Windows.Forms.MouseEventArgs) Handles PageLayout.MouseUp
      Dim oPrinterOverlay As PrinterInteractiveOverLay = mGetPrinterOverlay()
      Try
         Dim oLegend As LegendPrinterLayer = CType(oPrinterOverlay.PrinterLayers("Legend"), LegendPrinterLayer)

         With moLegendForm
            .txtLegendHeight.Text = oLegend.Height.ToString
            .txtLegendWidth.Text = oLegend.Width.ToString
            .txtLegendItemWidth.Text = oLegend.LegendItems(0).Width.ToString
            .txtCurrentItemFontSize.Text = oLegend.LegendItems(0).TextStyle.Font.Size.ToString
            .txtCurrentImageWidth.Text = oLegend.LegendItems(0).ImageWidth.ToString
            'oLegend.Height = 100
            'oLegend.Width = 500
            Dim oSize As RectangleShape = oLegend.GetPosition()
            .txtLegendHeight.Text = oSize.Height.ToString
            .txtLegendWidth.Text = oSize.Width.ToString

         End With
      Catch
      End Try

   End Sub
End Class
